/*
 * xrick/src/game.c
 *
 * This source code file is part of the xrick distribution, and as such is
 * copyrighted. Please read the README file in the root directory of the
 * distribution for information about copyrights, trademarks, license,
 * disclaimers, etc.
 */

#include <stdlib.h>

#include "system.h"
#include "game.h"

#include "draw.h"
#include "maps.h"
#include "ents.h"
#include "e_rick.h"
#include "e_sbonus.h"
#include "e_them.h"
#include "screens.h"
#include "rects.h"
#include "scroller.h"
#include "control.h"

U8 isave_frow;

U8 game_period = 0;
U8 game_waitevt = FALSE;
rect_t *game_rects = NULL;

U8 game_lives = 0;
U8 game_bombs = 0;
U8 game_bullets = 0;
U32 game_score = 0;

U16 game_map = 0;
U16 game_submap = 0;

U8 game_dir = 0;
U8 game_chsm = FALSE;

U8 game_cheat1 = 0;
U8 game_cheat2 = 0;
U8 game_cheat3 = 0;

/*
 * local typedefs
 */
typedef enum {
  IMAIN, IMAP,
  PAUSE_PRESSED1, PAUSED, PAUSE_PRESSED2,
  PLAY0, PLAY1, PLAY2, PLAY3,
  CHAIN_SUBMAP, CHAIN_MAP, CHAIN_END,
  SCROLL_UP, SCROLL_DOWN,
  RESTART, GAMEOVER, GETNAME, EXIT
} game_state_t;

static game_state_t game_state;

/*
 * prototypes
 */
static void frame(void);
static void init0(void);
static void init1(void);
static void play0(void);
static void play3(void);
static void restart(void);
static void isave(void);
static void irestore(void);

/*
 * Cheats
 */
void
game_toggleCheat(U8 nbr)
{
  if (game_state != IMAIN && game_state != IMAP && game_state != GAMEOVER &&
      game_state != GETNAME && game_state != EXIT) {
    switch (nbr) {
    case 1:
      game_cheat1 = ~game_cheat1;
      game_lives = 6;
      game_bombs = 6;
      game_bullets = 6;
      break;
    case 2:
      game_cheat2 = ~game_cheat2;
      break;
    case 3:
      game_cheat3 = ~game_cheat3;
      break;
    } 
    draw_infos();
    /* FIXME this should probably only raise a flag ... */
    sysvid_update(GAME_SCREEN);
  }
}

/*
 * Main loop
 *
 */
void
game_run(void)
{
  U32 tm, tmx;

  game_period = sysarg_args_period ? sysarg_args_period : GAME_PERIOD;

  tm = sys_gettime();
  game_state = IMAIN;

#ifdef DEBUG
  /* perf. test loop */
  {
    int i;
    tmx = sys_gettime();
    game_rects = GAME_SCREEN;
    for (i = 0; i < 100; i++)
      sysvid_update(game_rects);
    tm = sys_gettime();
    sys_printf("xrick/mainloop: period-min = %d\n", (tm-tmx)/100);
  }
#endif

  /* main loop */
  while (game_state != EXIT) {

    /* timer */
    tmx = tm; tm = sys_gettime(); tmx = tm - tmx;
    if (tmx < game_period) sys_sleep(game_period - tmx);

    /* video */
    sysvid_update(game_rects);

    /* sound */

    /* events */
    if (game_waitevt)
      sysevt_wait();  /* wait for an event */
    else
      sysevt_poll();  /* process events (non-blocking) */

    /* frame */
    frame();

  }

}

/*
 * Prepare frame
 *
 */
static void
frame(void)
{
  while (1) {

    switch (game_state) {
    
    case IMAIN:                                 /* MAIN INTRO */
      switch (screen_imain()) {
      case SCREEN_DONE:
	init0();
	game_state = IMAP;
	break;
      case SCREEN_EXIT:
	game_state = EXIT;
	break;
      }
      return;

    case IMAP:                                  /* MAP INTRO */
      switch (screen_imap()) {
      case SCREEN_DONE:
	init1();
	game_waitevt = FALSE;
	game_state = PLAY0;
	break;
      case SCREEN_EXIT:
	game_state = EXIT;
	break;
      }
      return;

    case PAUSE_PRESSED1:                        /* PAUSE PRESSED */
      if (!(control_status & CONTROL_PAUSE))
	game_state = PAUSED;
      return;

    case PAUSED:                                /* GAME PAUSED */
      if (control_status & CONTROL_PAUSE)
	game_state = PAUSE_PRESSED2;
      if (control_status & CONTROL_EXIT)
	game_state = EXIT;
      return;

    case PAUSE_PRESSED2:                        /* PAUSED PRESSED */
      if (!(control_status & CONTROL_PAUSE)) {
	game_state = PLAY2;
	game_waitevt = FALSE;
	syssnd_resume();
      }
      return;

    case PLAY0:
      play0();
      break;
    
    case PLAY1:
      if (control_status & CONTROL_PAUSE) {
	syssnd_pause();
	game_waitevt = TRUE;
	game_state = PAUSE_PRESSED1;
      }
      else
	game_state = PLAY2;
      break;
    
    case PLAY2:
      if E_RICK_STTST(E_RICK_STDEAD) {
        if (game_cheat1 || --game_lives)
	  game_state = RESTART;
        else
	  game_state = GAMEOVER; /* 0F */
      }
      else if (game_chsm)
	game_state = CHAIN_SUBMAP;
      else
	game_state = PLAY3;
      break;
    
    case PLAY3:
      play3();
      return;

    case CHAIN_SUBMAP:                          /* CHAIN SUBMAP */
      if (map_chain())
	game_state = CHAIN_END;
      else {
	game_bullets = 0x06;
	game_bombs = 0x06;
	game_map++;
	
	if (game_map == 0x04) {
	  /* reached end of game */
	  /* FIXME @292?*/
	}
	
	game_state = CHAIN_MAP;
      }
      break;
    
    case CHAIN_MAP:                             /* CHAIN MAP */
      switch (screen_imap()) {
      case SCREEN_DONE:
	if (game_map >= 0x04) {
	  /* reached end of game */
	  game_state = GAMEOVER;
	  sysarg_args_map = 0;
	  sysarg_args_submap = 0;
	}
	else {
	  ent_ents[1].x = map_maps[game_map].x;
	  ent_ents[1].y = map_maps[game_map].y;
	  map_frow = map_maps[game_map].row;
	  game_submap = map_maps[game_map].submap;
	  game_state = CHAIN_END;
	}
	break;
      case SCREEN_EXIT:
	game_state = EXIT;
	break;
      default:
	return;
      }
      break;

    case CHAIN_END:                             /* CHAIN */
      map_init();
      isave();
      ent_clprev();
      draw_map();
      draw_status();
      game_rects = GAME_SCREEN;
      game_state = PLAY3;
      return;

    case SCROLL_UP:                             /* SCROLLUP */
      switch (scroll_up()) {
      case SCROLL_RUNNING:
	return;
      case SCROLL_DONE:
	game_state = PLAY0;
	break;
      }
      break;
    
    case SCROLL_DOWN:                           /* SCROLL_DOWN */
      switch (scroll_down()) {
      case SCROLL_RUNNING:
	return;
      case SCROLL_DONE:
	game_state = PLAY0;
	break;
      }
      break;

    case RESTART:                               /* RESTART AFTER DEATH */
      restart();
      game_state = PLAY0;
      return;

    case GAMEOVER:                              /* GAMEOVER SCREEN */
      switch (screen_gameover()) {
      case SCREEN_DONE:
	game_state = GETNAME;
	return;
      case SCREEN_EXIT:
	game_state = EXIT;
	break;
      }

    case GETNAME:                               /* GET NAME - HALL OF FAME */
      switch (screen_getname()) {
      case SCREEN_DONE:
	game_state = IMAIN;
	return;
      case SCREEN_EXIT:
	game_state = EXIT;
	break;
      }

    case EXIT:
      return;

    }
  }
}


/*
 * init0
 *
 */
static void
init0(void)
{
  E_RICK_STRST(0xff);

  game_lives = 6;
  game_bombs = 6;
  game_bullets = 6;
  game_score = 0;

  game_map = sysarg_args_map;
    
  ent_ents[1].x = map_maps[game_map].x;
  ent_ents[1].y = map_maps[game_map].y;
  ent_ents[1].w = 0x18;
  ent_ents[1].h = 0x15;
  ent_ents[1].n = 0x01;
  ent_ents[1].sprite = 0x01;
  ent_ents[ENT_ENTSNUM].n = 0xFF;

  map_resetMarks();
}


/*
 * init1
 *
 */
static void
init1(void)
{
  U8 i;

  sysvid_clear();

  if (sysarg_args_submap != 0) {
    game_submap = sysarg_args_submap;

    /* figure out frow (rather dirty) */
    i = 0;
    while (i < MAP_NBR_CONNECT &&
	   (map_connect[i].submap != game_submap ||
	    map_connect[i].dir != RIGHT))
      i++;
    map_frow = map_connect[i].rowin - 0x10;
    ent_ents[1].y = 0x10 << 3;
  }
  else {
    game_submap = map_maps[game_map].submap;
    map_frow = map_maps[game_map].row;
  }
  /* done */

  map_init();
  isave();
  draw_map();
  draw_status();
  draw_infos();
  game_rects = GAME_SCREEN;
}

/*
 * play0
 *
 */
static void
play0(void)
{
  /* clear */
  game_rects = NULL;

  /* end */
  if (control_status & CONTROL_END) {
    game_state = GAMEOVER; /* FF */
    return;
  }

  /* exit */
  if (control_last == CONTROL_EXIT) {
    game_state = EXIT;
    return;
  }

  /* action */
  ent_action();
  /* call 0270 */
  e_them_rndseed++;

  /* speed bonus counter (01A6) */
  if (e_sbonus_counting && --e_sbonus_counter == 0) {
    e_sbonus_counter = 0x1e;
    if (e_sbonus_bonus)
      e_sbonus_bonus--;
  }

  game_state = PLAY1;
}


/*
 * play3
 *
 */
static void
play3(void)
{
  /* FIXME should use constants here */
  static rect_t rcnt = { 0x28, 0x08, 0xf0, 8, NULL };
  static rect_t *r;

  draw_clearStatus();

  ent_draw();

  /*
    snd_something();
    v_sync;
    snd_something();
    v_sync;
  */

  /*
   * original rick just calls d_cnt_updbb but they have another way of
   * implementing clipping (via a call to 0d2f in 0a54). We have to do
   * the clipping by ourselves -> keep redrawing everything.
   */
  draw_status();
  r = &rcnt; r->next = ent_rects;
  game_rects = r;

  if (!E_RICK_STTST(E_RICK_STZOMBIE)) {
    if (ent_ents[1].y >= 0xCC) {
      game_state = SCROLL_UP;
      return;
    }
    if (ent_ents[1].y <= 0x60) {
      game_state = SCROLL_DOWN;
      return;
    }
  }  

  game_state = PLAY0;
}


/*
 * restart
 *
 */
static void
restart(void)
{
  E_RICK_STRST(E_RICK_STDEAD|E_RICK_STZOMBIE);

  game_bullets = 6;
  game_bombs = 6;

  ent_ents[1].n = 1;

  irestore();
  map_init();
  isave();
  ent_clprev();
  draw_map();
  draw_status();
  game_rects = GAME_SCREEN;
}


/*
 * isave (0bbb)
 *
 */
static void
isave(void)
{
  e_rick_save();
  isave_frow = map_frow;
}


/*
 * irestore (0bdc)
 *
 */
static void
irestore(void)
{
  e_rick_restore();
  map_frow = isave_frow;
}


/* eof */
